@@ -14,8 +14,11 @@ fn can_solve_problem_using_full_config() {
     let reader = BufReader::new(File::open("../examples/data/config/config.full.json").unwrap());
     // TODO override termination to avoid test timeout on CI
     let mut config = read_config(reader).unwrap();
+    if let Some(initial) = config.evolution.as_mut().and_then(|evolution| evolution.initial.as_mut()) {
+        initial.alternatives.max_size = 1;
+    }
     if let Some(termination) = config.termination.as_mut() {
-        termination.max_time = Some(10);
+        termination.max_generations = Some(1);
     }
 
     let (solution, _, _) = create_builder_from_config(problem, &config).unwrap().build().unwrap().solve().unwrap();
